/* $Id: multisign.c,v 1.9.2.1 2015/03/27 09:22:13 tm Exp $
 *
 * PDFlib PLOP DS:
 * Demonstrate mass signature with optional PKCS#11 session handling
 *
 */

#include <stdio.h>

#include "ploplib.h"

int
main(int argc, char **argv)
{
    PLOP *plop;

    /* Basic signature options */
    static const char sign_opts[] =
        "sigtype=cms "      /* use sigtype=cades for PAdES signature */
        "ltv=try "          /* use ltv=full to ensure LTV status */
#if defined(USE_PKCS11)
        "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
        "engine=builtin digitalid={filename=demorsa2048.p12} password={demo} "
#endif
    ;

    /* This is where input files live. Adjust as necessary. */
    const char *searchpath = "../data";

    if (argc < 2)
    {
        printf("usage: multisign <infile>...\n");
        return 2;
    }

    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(2);
    }

    PLOP_TRY(plop)
    {
	int doc;
	int i;
	char optlist[1024];

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(plop, optlist);

	if (PLOP_prepare_signature(plop, sign_opts) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	for (i = 1; i < argc; i++)
	{
	    char signed_name[256];

	    printf("Signing '%s'...\n", argv[i]);

            if ((doc = PLOP_open_document(plop, argv[i], 0, "")) == -1)
            {
                printf("Error: %s\n", PLOP_get_errmsg(plop));
		continue;
            }

            sprintf(signed_name, "%s-multisigned.pdf", argv[i]);
            sprintf(optlist, "input=%d", doc);
            if (PLOP_create_document(plop, signed_name, 0, optlist) == -1)
            {
                printf("Error: %s\n", PLOP_get_errmsg(plop));

#if defined(USE_PKCS11)
		/* The PKCS#11 session may have expired;
		 * restart and try again.
		 */
		if (PLOP_prepare_signature(plop, sign_opts) == -1)
		{
		    printf("Error: %s\n", PLOP_get_errmsg(plop));
		    PLOP_delete(plop);
		    return(2);
		}
		if (PLOP_create_document(plop, signed_name, 0, optlist) == -1)
		{
		    printf("Error: %s\n", PLOP_get_errmsg(plop));
		    PLOP_delete(plop);
		    return(2);
		}
#endif
            }

            PLOP_close_document(plop, doc, "");
	}
    }
    PLOP_CATCH(plop)
    {
	printf("PLOP exception occurred in multisign:\n");
	printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
		PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	PLOP_delete(plop);
	return(2);
    }

    PLOP_delete(plop);

    return(0);
}
